/* -*- Mode:C++; c++-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Marco Guastella alias Vasta 
 * Web page:<www.ragnu.it> 
 * Email:<vasta@ragnu.it>
   Date last update: 05/12/2024
*/

#include "json.h"

namespace vampiria { namespace json {

vmp::str japi(vmp::str jtype)
{
    vmp::str ret;
    vmp::unicode::str_write(&ret,"json::%s",jtype.c_str());
    return ret;
}

vmp::str japi_type_list(vmp::str jtype)
{
    vmp::str ret;
    vmp::unicode::str_write(&ret,"[%s]",jtype.c_str());
    return ret;
}

vmp::str japi_type_table(vmp::str jtype)
{
    vmp::str ret;
    vmp::unicode::str_write(&ret,"{%s}",jtype.c_str());
    return ret;
}

void japi_void(json::JsonObj *obj)
{
    vmp::except_check_pointer((void *) obj,"json::japi_void(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("void"));
}

void japi_list(json::JsonObj *obj,vmp::str datatype)
{
    vmp::except_check_pointer((void *) obj,"json::japi_list(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi_type_list(datatype));
    jdata.set_text("datatype",datatype);
    json::JList list;
    jdata.new_list("list",&list,datatype);    
}

void japi_list(json::JsonObj *obj,json::JList *list)
{
    vmp::except_check_pointer((void *) obj,"json::japi_void(obj=Null)");
    vmp::except_check_pointer((void *) list,"json::japi_list(jlist=Null)");
    json::JData jdata;
    vmp::str datatype=list->jdata_type();
    jdata.new_data(obj,json::japi_type_list(datatype));
    jdata.set_text("datatype",datatype);
    jdata.set_list("list",list); 
}

void japi_table(json::JsonObj *obj,vmp::str datatype)
{
    vmp::except_check_pointer((void *) obj,"json::japi_table(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi_type_table(datatype));
    jdata.set_text("datatype",datatype);
    json::JTable table;
    jdata.new_table("table",&table,datatype);    
}

void japi_table(json::JsonObj *obj,json::JTable *table)
{
    vmp::except_check_pointer((void *) obj,"json::japi_table(obj=Null)");
    vmp::except_check_pointer((void *) table,"json::japi_table(table=Null)");
    json::JData jdata;
    vmp::str datatype=table->jdata_type();
    jdata.new_data(obj,json::japi_type_table(datatype));
    jdata.set_text("datatype",datatype);
    jdata.set_table("table",table); 
}

void japi_oper(json::JsonObj *obj,vmp::str id,json::JsonObj *input,vmp::Buf *bin)
{
    vmp::except_check_pointer((void *) obj,"json::japi_oper(obj=Null)");
    json::JData jdata,tmp;
    try
    {
        tmp.set(input);
    }
    catch(vmp::exception &x)
    {
        vmp::except_s("json::japi_oper(input='bad object')");
    }
    jdata.new_data(obj,json::japi("oper"));
    jdata.set_text("id",id);
    if(bin == 0)
        jdata.set_text("bin","");
    else
    {
        bin->index();
        jdata.set_text("bin",bin->read_xstr(bin->size()));
    }
    jdata.set_subtype("input",input);    

}

void japi_oper_kill(json::JsonObj *obj,vmp::str id)
{
    vmp::except_check_pointer((void *) obj,"json::japi_oper_kill(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("operkill"));
    jdata.set_text("id",id);
}

void japi_oper_data(json::JsonObj *obj,vmp::str id,json::JsonObj *data,vmp::Buf *bin)
{
    vmp::except_check_pointer((void *) obj,"json::japi_oper_data(obj=Null)");
    json::JData jdata,tmp;
    try
    {
        tmp.set(data);
    }
    catch(vmp::exception &x)
    {
        vmp::except_s("json::japi_oper_data(data='bad object')");
    }
    jdata.new_data(obj,json::japi("operdata"));
    jdata.set_text("id",id);
    if(bin == 0)
        jdata.set_text("bin","");
    else
    {
        bin->index();
        jdata.set_text("bin",bin->read_xstr(bin->size()));
    }
    jdata.set_subtype("data",data);
}

void japi_oper_push(json::JsonObj *obj,vmp::str id,json::JsonObj *data,vmp::Buf *bin)
{
    vmp::except_check_pointer((void *) obj,"json::japi_oper_push(obj=Null)");
    json::JData jdata,tmp;
    try
    {
        tmp.set(data);
    }
    catch(vmp::exception &x)
    {
        vmp::except_s("json::japi_oper_push(data='bad object')");
    }
    jdata.new_data(obj,json::japi("operpush"));
    jdata.set_text("id",id);
    if(bin == 0)
        jdata.set_text("bin","");
    else
    {
        bin->index();
        jdata.set_text("bin",bin->read_xstr(bin->size()));
    }
    jdata.set_subtype("data",data);
}

void japi_oper_close(json::JsonObj *obj,vmp::str id,vmp_int code,vmp::str msg)
{
    vmp::except_check_pointer((void *) obj,"json::japi_oper_close(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("operclose"));
    jdata.set_text("id",id);
    jdata.set_integer("code",code);
    jdata.set_text("msg",msg);    
}

void japi_status(json::JsonObj *obj)
{
    vmp::except_check_pointer((void *) obj,"json::japi_status(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("status"));
}

void japi_status_update(json::JsonObj *obj,vmp::str key,json::JsonObj *data)
{
    vmp::except_check_pointer((void *) obj,"json::japi_status_update(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("statusupdate"));
    jdata.set_text("key",key);
    jdata.set_subtype("data",data);
}

void japi_status_cancel(json::JsonObj *obj,vmp::str key)
{
    vmp::except_check_pointer((void *) obj,"json::japi_status_cancel(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("statuscancel"));
    jdata.set_text("key",key);
}

void japi_login(json::JsonObj *obj,vmp::str user,vmp::str password,vmp_bool md5_password)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_login(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("login"));
    jdata.set_text("user",user);
    jdata.set_text("password",password);
    jdata.set_bool("md5_password",md5_password);  
}

vmp::str japi_login_b64_e(vmp::str user,vmp::str password,vmp_bool md5_password)
{
    json::Json j;
    json::japi_login(j.root(),user,password,md5_password);
    return vmp::unicode::b64_encode(j.json_str());
}

void japi_login_b64_d(json::Json *json,vmp::str etoken)
{
    vmp::except_check_pointer((void *) json,"json::japi_login_b64_d(json=Null)");    
    try
    {
        json::JData jdata;
        jdata.set(json->parse_from_str(vmp::unicode::b64_decode(etoken)));
        if(jdata.jtype() != json::japi("login"))
            vmp::except_s("");
    }
    catch(vmp::exception &x)
    {
        vmp::except("json::japi_login_b64_d(etoken=%s) bad value",etoken.c_str());
    }
}

void japi_passwd(json::JsonObj *obj,vmp::str oldpassword,vmp::str password,vmp_bool md5_password)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_passwd(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("passwd"));
    jdata.set_text("oldpassword",oldpassword);
    jdata.set_text("password",password);
    jdata.set_bool("md5_password",md5_password); 
}

void japi_userlist(json::JsonObj *obj)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_userlist(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("userlist"));
}

void japi_userinfo(json::JsonObj *obj,vmp::str user,vmp_uint permits,vmp_bool only_localhost)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_userinfo(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("userinfo"));
    jdata.set_text("user",user);
    jdata.set_integer_range("permits",0,vmp::INTMAX,(vmp_int)permits);
    jdata.set_bool("only_localhost",only_localhost);
}

void japi_useradd(json::JsonObj *obj,vmp::str user,vmp::str password,vmp_uint permits,vmp_bool only_localhost,vmp_bool md5_password)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_useradd(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("useradd"));
    jdata.set_text("user",user);
    jdata.set_text("password",password);
    jdata.set_integer_range("permits",0,vmp::INTMAX,(vmp_int)permits);
    jdata.set_bool("only_localhost",only_localhost);
    jdata.set_bool("md5_password",md5_password);
}

void japi_usermod(json::JsonObj *obj,vmp::str user,vmp_uint permits,vmp_bool only_localhost)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_usermod(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("usermod"));
    jdata.set_text("user",user);
    jdata.set_integer_range("permits",0,vmp::INTMAX,(vmp_int)permits);
    jdata.set_bool("only_localhost",only_localhost);
}

void japi_userpasswd(json::JsonObj *obj,vmp::str user,vmp::str password,vmp_bool md5_password)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_userpasswd(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("userpasswd"));
    jdata.set_text("user",user);
    jdata.set_text("password",password);
    jdata.set_bool("md5_password",md5_password);
}

void japi_userdel(json::JsonObj *obj,vmp::str user)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_userdel(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("userdel"));
    jdata.set_text("user",user);
}

void japi_peerlist(json::JsonObj *obj)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_peerlist(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("peerlist"));
}

void japi_peerinfo(json::JsonObj *obj,vmp::str fingerprint,vmp::str subject,vmp_uint permits)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_peerinfo(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("peerinfo"));
    jdata.set_text("fingerprint",fingerprint);
    jdata.set_text("subject",subject);
    jdata.set_integer_range("permits",0,vmp::INTMAX,(vmp_int)permits);
}

void japi_peeradd(json::JsonObj *obj,vmp::str fingerprint,vmp::str subject,vmp_uint permits)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_peeradd(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("peeradd"));
    jdata.set_text("fingerprint",fingerprint);
    jdata.set_text("subject",subject);
    jdata.set_integer_range("permits",0,vmp::INTMAX,(vmp_int)permits);
}

void japi_peermod(json::JsonObj *obj,vmp::str fingerprint,vmp_uint permits)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_peermod(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("peermod"));
    jdata.set_text("fingerprint",fingerprint);
    jdata.set_integer_range("permits",0,vmp::INTMAX,(vmp_int)permits);
}

void japi_peerdel(json::JsonObj *obj,vmp::str fingerprint)
{
    vmp::except_check_pointer((void *) obj,"web::pkg::japi_peerdel(obj=Null)");
    json::JData jdata;
    jdata.new_data(obj,json::japi("peerdel"));
    jdata.set_text("fingerprint",fingerprint);
}

}}

